unit QRVLabel;

{(C)1996-1997 by Marcin Qfel Zaleski}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Quickrep;

type
  TQRVerticalLabel = class(TQRCustomControl)
    protected
      procedure Paint; override;
      procedure Print(X,Y:Integer); override;
    public
      constructor Create(AOwner:TComponent); override;
    published
      property AutoSize;
      property Caption;
      property Color;
      property Font;
      property Height;
      property OnPrint;
      property ShowHint;
      property Transparent;
      property Width;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Qfel',[TQRVerticalLabel]);
end;

constructor TQRVerticalLabel.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Font.Name := 'Arial';
end;

procedure TQRVerticalLabel.Paint;
var
  LogFont  : TLogFont;
  HNewFont : HFont;
  HOldFont : HFont;
  R        : TRect;
  Options  : Word;
begin
  if AutoSize then
    begin
      Width := Canvas.TextHeight(Caption);
      Height := Canvas.TextWidth(Caption);
      Options := 0;
    end
  else
    Options := ETO_CLIPPED;
  if Transparent then
    begin
      SetBkMode(Canvas.Handle,Windows.TRANSPARENT)
    end
  else
    begin
      SetBkMode(Canvas.Handle,OPAQUE);
      Options := Options or ETO_OPAQUE;
    end;
  Canvas.Brush.Color := Color;
  SetTextColor(Canvas.Handle,Font.Color);
  GetObject(Font.Handle,SizeOf(TLogFont),@LogFont);
  with LogFont do
    begin
      lfEscapement :=  900;
      lfOutPrecision := OUT_TT_ONLY_PRECIS;
    end;
  HNewFont := CreateFontIndirect(LogFont);
  HOldFont := SelectObject(Canvas.Handle,HNewFont);
  SetTextAlign(Canvas.Handle,TA_TOP or TA_RIGHT or TA_NOUPDATECP);
  R := Rect(0,0,Width,Height);
  ExtTextOut(Canvas.Handle,0,0,Options,@R,@Caption[1],Length(Caption),nil);
  HNewFont := SelectObject(Canvas.Handle,HOldFont);
  DeleteObject(HNewFont);
end;

procedure TQRVerticalLabel.Print(X,Y:Integer);
var
  LogFont  : TLogFont;
  HNewFont : HFont;
  HOldFont : HFont;
  R        : TRect;
  Options  : Word;
begin
  if AutoSize then
    Options := 0
  else
    Options := ETO_CLIPPED;
  if Transparent then
    begin
      SetBkMode(QRPrinter.Canvas.Handle,Windows.TRANSPARENT)
    end
  else
    begin
      SetBkMode(QRPrinter.Canvas.Handle,OPAQUE);
      Options := Options or ETO_OPAQUE;
    end;
  QRPrinter.Canvas.Brush.Color := Color;
  SetTextColor(QRPrinter.Canvas.Handle,Font.Color);
  GetObject(Font.Handle,SizeOf(TLogFont),@LogFont);
  with LogFont do
    begin 
      {Next two lines required by Quick Report 1.1}
      lfHeight := ParentReport.YPos(lfHeight);
      lfWidth := ParentReport.XPos(lfWidth);
      lfEscapement :=  900;
      lfOutPrecision := OUT_TT_ONLY_PRECIS;
    end;
  HNewFont := CreateFontIndirect(LogFont);
  HOldFont := SelectObject(QRPrinter.Canvas.Handle,HNewFont);
  SetTextAlign(QRPrinter.Canvas.Handle,TA_TOP or TA_RIGHT or TA_NOUPDATECP);
  with ParentReport do
    begin
      R := Rect(XPos(Left),YPos(Top),XPos(Left+Width),YPos(Top+Height));
      ExtTextOut(QRPrinter.Canvas.Handle,XPos(Left),YPos(Top),Options,@R,@Caption[1],
                                                         Length(Caption),nil);
    end;
  HNewFont := SelectObject(QRPrinter.Canvas.Handle,HOldFont);
  DeleteObject(HNewFont);
end;

end.
